@@ -13,11 +13,16 @@ from . import visualization as viz
 from .io.fitsmanager import FitsManager
 from .fluxes import ApertureFluxes
 from .telescope import Telescope
-from .block import Block
-from .sequence import Sequence
-from .image import Image
+from .core.block import Block
+from .core.sequence import Sequence
+from .core.image import Image
 from .observation import Observation
 from .observations import Observations
+from .core import source
+from .tutorials import example_image
 
 from pkg_resources import get_distribution
 __version__ = get_distribution('prose').version
+
+# TODO: update Telescope "names" fields
+# TODO: document custom Image using _get_data_header
\ No newline at end of file
