@@ -783,7 +783,7 @@ def corner_text(text, loc=(0.05, 0.05), va="bottom", ha='left', fontsize=12, c="
 
 
 def plot_systematics_signal(x, y, systematics, signal=None, ylim=None, offset=None, figsize=(6, 7), signal_label=None):
-    """Plot a systematics and signal model over data. systeamtics + signal is plotted on top, signal alone on detrended 
+    """Plot a systematics and signal model over data. systematics + signal is plotted on top, signal alone on detrended
     data on bottom
 
     Parameters
@@ -841,9 +841,11 @@ class HandlerEllipse(HandlerPatch):
         return [p]
 
 
-def circles_legend(colors, texts):
+def circles_legend(colors, texts, ax=None):
+    if ax is None:
+        ax = plt.gca()
     c = [mpatches.Circle((0.5, 0.5), radius = 0.25, fill=None, ec=colors[i]) for i in range(len(texts))]
-    plt.legend(c, texts , bbox_to_anchor=(1, 1.05), loc='upper right', ncol=3, handler_map={mpatches.Circle: HandlerEllipse()}, frameon=False)
+    ax.legend(c, texts , bbox_to_anchor=(1, 1.05), loc='upper right', ncol=3, handler_map={mpatches.Circle: HandlerEllipse()}, frameon=False)
 
 
 def plot_signal(x, y, label=None, **kwargs):
